"""
A simple class for making parser filters, and some demo filters.
"""

import string

# --- ParserFilter

class ParserFilter:
    "A no-op parser filter base class that passes on all events unchanged."

    def __init__(self,next):
        self.next=next
    
    def characters(self, ch, start, length):
        self.next.characters(ch,start,length)
        
    def endDocument(self):
        self.next.endDocument()
        
    def endElement(self, name):
        self.next.endElement(name)
        
    def ignorableWhitespace(self, ch, start, length):
        self.next.ignorableWhitespace(ch,start,length)

    def processingInstruction(self, target, data):
        self.next.processingInstruction(target,data)

    def setDocumentLocator(self, locator):
        self.next.setDocumentLocator(locator)

    def startDocument(self):
        self.next.startDocument()

    def startElement(self, name, atts):
        self.next.startElement(name,atts)

# --- BlockDataFilter

class BlockDataFilter(ParserFilter):
    """A simple filter that ensures that all character data are reported in
    single blocks."""
 
    def __init__(self,next):
        ParserFilter.__init__(self,next)
        self.block=""
        self.ignorable=0

    def characters(self, ch, start, length):
        self.block=self.block+ch[start:start+length]

    def ignorableWhitespace(self, ch, start, length):
        self.ignorable=1
        self.block=self.block+ch[start:start+length]

    def endDocument(self):
        self.emitBlock()
        self.next.endDocument()
        
    def endElement(self, name):
        self.emitBlock()
        self.next.endElement(name)
        
    def processingInstruction(self, target, data):
        self.emitBlock()
        self.next.processingInstruction(target,data)

    def startElement(self, name, atts):
        self.emitBlock()
        self.next.startElement(name,atts)

    # Internal methods

    def emitBlock(self):
        if self.block!="":
            if self.ignorable:
                self.next.ignorableWhitespace(self.block,0,len(self.block))
            else:
                self.next.characters(self.block,0,len(self.block))
            self.block=""
        
        self.ignorable=0
        
# --- WSFilter

def strip_lines(str):
    res=""
    for line in string.split(str,"\n"):
        line=string.strip(line)
        if line!="":
            res=res+line+"\n"

    return res
    
class WSFilter(ParserFilter):
    "A simple filter that can strip whitespace from character data."

    LEADING=0     # Strip leading space
    TRAILING=1    # Strip trailing space
    BOTH=2        # Obvious, isn't it?
    NORMALIZE=3   # Every WS sequence replaced by a single space
    STRIP_LINES=4 # Strips leading and trailing space on each line
    
    def __init__(self,next,behaviour):
        ParserFilter.__init__(self,next)

        if behaviour==WSFilter.LEADING:
            self.filter=string.lstrip
        elif behaviour==WSFilter.TRAILING:
            self.filter=string.rstrip
        elif behaviour==WSFilter.BOTH:
            self.filter=string.strip
        elif behaviour==WSFilter.NORMALIZE:
            self.filter=lambda str: string.join(string.split(str))
        elif behaviour==WSFilter.STRIP_LINES:
            self.filter=strip_lines

    def characters(self, ch, start, length):
        ch=self.filter(ch[start:start+length])
        self.next.characters(ch,0,len(ch))

    def ignorableWhitespace(self, ch, start, length):
        ch=self.filter(ch[start:start+length])
        self.next.characters(ch,0,len(ch))
